#!/usr/bin/env python

import os
from OpenGL.GL import *
from OpenGL.GL.EXT.framebuffer_object import *
import pygame, pygame.image
from pygame.locals import *

from fbo import FBO
from texture import Texture
from shader import Shader
from light import Light
from material import Material
from timer import Timer
from functions import drawcube, resize, initvbocube, drawquad

from math import sin, cos
from optparse import OptionParser

SHADERS_DIR = os.path.join('shaders', 'dof2')
NUM_FBOS = 4
DEFAULT_DISTANCE = 9.0
DEFAULT_RANGE = 6.0


def main():
	global movz, roty, tex0, tex1, lights

	interval = 0.0
	movz = 0.0; roty = 0.0
	move = False; rotate = True; bg = True
	blurmode = False
#	withVBO = False
	focalDistance = DEFAULT_DISTANCE
	focalRange = DEFAULT_RANGE

# Parser class
	parser = OptionParser()
	parser = OptionParser(usage="usage: %prog [options]", version="%prog 0.1")
	parser.add_option("-x", "--width", type="int", dest="width", default=640,
					help="To set the window width (default: %default)")
	parser.add_option("-y", "--height", type="int", dest="height", default=480,
					help="To set the window height (default: %default)")
	parser.add_option("-f", "--fullscreen", action="store_true", dest="fullscreen", default=False,
					help="To set fullscreen mode (default: %default)")
	(options, args) = parser.parse_args()

# PyGame init
	video_flags = OPENGL|DOUBLEBUF

	if options.fullscreen:
		video_flags |= FULLSCREEN
    
	pygame.init()
	pygame.display.set_caption('PyGLSL_DoF2 by Encelo')
	surface = pygame.display.set_mode((options.width, options.height), video_flags)

# OpenGL basic init
	glShadeModel(GL_SMOOTH)
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)
	glClearColor(0.0, 0.0, 0.0, 1.0)
	glClearDepth(1.0)
	glEnable(GL_DEPTH_TEST)
	glEnable(GL_TEXTURE_2D)
	resize((640,480))

# OpenGL classes init
	fbo = []
	temp = FBO(options.width, options.height)
	temp.attach_render(GL_DEPTH_COMPONENT24)
	temp.attach_texture(GL_RGBA16F_ARB)
	fbo.append(temp)
	for i in range(NUM_FBOS):
		temp = FBO(options.width/2, options.height/2)
		temp.attach_texture(GL_RGBA16F_ARB)
		fbo.append(temp)
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0)
	glDrawBuffer(GL_BACK)

	first      = Shader(os.path.join(SHADERS_DIR, 'first.vert'),  os.path.join(SHADERS_DIR, 'first.frag'))
	second     = Shader(os.path.join(SHADERS_DIR, 'second.vert'), os.path.join(SHADERS_DIR, 'second.frag'))
	third      = Shader(os.path.join(SHADERS_DIR, 'third.vert'),  os.path.join(SHADERS_DIR, 'third.frag'))
	fourth     = Shader(os.path.join(SHADERS_DIR, 'fourth.vert'), os.path.join(SHADERS_DIR, 'fourth.frag'))
	fifth      = Shader(os.path.join(SHADERS_DIR, 'fifth.vert'),  os.path.join(SHADERS_DIR, 'fifth.frag'))
	renderblur = Shader(os.path.join(SHADERS_DIR, 'fifth.vert'),  os.path.join(SHADERS_DIR, 'renderblur.frag'))
	third.update_uniform('Width', options.width*2)
	fourth.update_uniform('Height', options.height*2)
	fifth.update_uniform('Tex0', 0)
	fifth.update_uniform('Tex1', 1)
	glUseProgram(0)

	tex0 = Texture(os.path.join('textures', 'tex0.jpg'))
	tex1 = Texture(os.path.join('textures', 'tex1.png'))

	timer = Timer()

# Event Cycle
	while 1:
		event = pygame.event.poll()
		if event.type == QUIT:
			break
		if event.type == KEYDOWN:
			if event.key == K_ESCAPE:
				return 0

			elif event.key == K_SPACE:
				move = not move
			elif event.key == K_r:
				rotate = not rotate
			elif event.key == K_b:
				bg = not bg
			elif event.key == K_m:
				blurmode = not blurmode
			elif event.key == K_p:
				print('focalDistance = %f' % focalDistance)
				print('focalRange = %f' % focalRange)
			elif event.key == K_KP7:
				focalDistance -= 0.5
			elif event.key == K_KP8:
				focalDistance = DEFAULT_DISTANCE
			elif event.key == K_KP9:
				focalDistance += 0.5
			elif event.key == K_KP4:
				focalRange -= 0.5
			elif event.key == K_KP5:
				focalRange = DEFAULT_RANGE
			elif event.key == K_KP6:
				focalRange += 0.5
			elif event.key == K_KP0:
				focalDistance = DEFAULT_DISTANCE
				focalRange = DEFAULT_RANGE
				movz = 0.0
				roty = 0.0
			elif event.key == K_1:
				focalDistance = 9.0
			elif event.key == K_2:
				focalDistance = 12.5
			elif event.key == K_3:
				focalDistance = 17.5
			elif event.key == K_4:
				focalDistance = 21.0
			elif event.key == K_5:
				focalDistance = 25.0
#			elif event.key == K_v:
#				withVBO = not withVBO


# Blitting on screen
		if move:
			movz += 0.0025 * interval
		if rotate:
			roty += 0.1 * interval

		# First pass: Scene rendering
		fbo[0].bind_in()
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
		glActiveTexture(GL_TEXTURE0)
		first.use()
		first.update_uniform("focalDistance", focalDistance)
		first.update_uniform("focalRange", focalRange)
		glLoadIdentity()
		glTranslatef(0.0, -2.0, -10.0)
		glRotatef(15.0, 1.0, 0.0, 0.0)

		#Brick Room
		if bg:
			tex0.bind()
			glPushMatrix()
			glTranslatef(0.0, 6.0, -10.0)
			glScalef(10.0, 10.0, 10.0)
			drawcube(True)
			glPopMatrix()

		# Rotating cubes
		tex1.bind()
		for i in range(5):
			glPushMatrix()
			glTranslatef(0.0, 0.0, 3.0 * sin(movz))
			glTranslatef(-2.0, 0.0, 0.0)
			glTranslatef(i * 2.0, 0.0, i * (-4.0))
			glRotatef(roty, 0.0, 1.0, 0.0)
			drawcube(False)
			glPopMatrix()

		glMatrixMode(GL_PROJECTION)
		glPushMatrix()
		glLoadIdentity()
		glMatrixMode(GL_MODELVIEW)
		glLoadIdentity()

		if not blurmode:
			# Second pass: downsampling
			fbo[1].bind_in()
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
			fbo[0].bind_tex()
			second.use()

			glViewport(0, 0, options.width/2, options.height/2)

			drawquad()


			# Third pass: Gaussian filtering along the X axis
			fbo[2].bind_in()
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
			fbo[1].bind_tex()
			third.use()

			drawquad()


			# Fourth pass: Gaussian filtering along the Y axis
			fbo[3].bind_in()
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
			fbo[2].bind_tex()
			fourth.use()

			drawquad()


			# Fifth pass: final compositing
			glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0)
			glDrawBuffer(GL_BACK)
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
			glActiveTexture(GL_TEXTURE0)
			fbo[0].bind_tex()
			glActiveTexture(GL_TEXTURE1)
			fbo[3].bind_tex()
			fifth.use()

			glViewport(0, 0, options.width, options.height)

			drawquad()

		else:
			fbo[0].bind_out()
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
			renderblur.use()

			drawquad()

		glMatrixMode(GL_PROJECTION)
		glPopMatrix()
		glMatrixMode(GL_MODELVIEW)


		pygame.display.flip()
		interval = timer.addframe()


if __name__ == '__main__':
	main()
