from OpenGL.GL import *
from OpenGL.raw.GL.constants import *
from OpenGL.GL.EXT.framebuffer_object import *
from OpenGL.GL.ARB.draw_buffers import *

class FBO(object):
	def __init__(self, width, height):
#		self.__max_color_attachments = glGetInteger(GL_MAX_COLOR_ATTACHMENTS_EXT) # FIXME: move in a glContext class
#		self.__max_draw_buffers = glGetInteger(GL_MAX_DRAW_BUFFERS_EXT) # FIXME: move in a glContext class
		self.__width = width
		self.__height = height

		self.__renders = []
		self.__textures = []
		self.__frameid = glGenFramebuffersEXT(1)
		self.__buffers = [] # For glDrawBuffers()

		# Seems like a wrapper bug, you can't write in the first buffer if you use glDrawBuffers()
		# Shaders have to be modified to write starting from gl_FragData[1]
		# self.__buffers.append(GL_NONE)


	def __del__(self):
		for id in self.__textures:
			glDeleteTextures(id)
		for id in self.__renders:
			glDeleteRenderbuffersEXT(1, id)
		glDeleteFramebuffersEXT(1, self.__frameid)


	def bind_in(self):
		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, self.__frameid)
		if len(self.__buffers) == 1:
			glDrawBuffer(self.__buffers[0])
		else:
			glDrawBuffers(len(self.__buffers), self.__buffers)


	def bind_out(self):
		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0)
		glDrawBuffer(GL_BACK)
		for i in range(len(self.__textures)):
			texunit = getattr(OpenGL.GL, 'GL_TEXTURE%d' % i)
			glActiveTexture(texunit)
			glBindTexture(GL_TEXTURE_2D, self.__textures[i])


	def bind_tex(self, num = 0):
		if num + 1 > len(self.__textures):
			raise Error

		glBindTexture(GL_TEXTURE_2D, self.__textures[num])


	def attach_render(self, iformat):
		if iformat == GL_DEPTH_COMPONENT24:
			attachment = GL_DEPTH_ATTACHMENT_EXT
		elif iformat == GL_STENCIL_INDEX:
			attachment = GL_STENCIL_ATTACHMENT_EXT

		id = glGenRenderbuffersEXT(1)
		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, self.__frameid)
		glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, id)
		glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, iformat, self.__width, self.__height)
		glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, attachment, GL_RENDERBUFFER_EXT, id)

		self.__renders.append(id)


	def attach_texture(self, iformat):
#		if len(self.__textures) > self.__max_color_attachments:
#			raise Error

		if iformat == GL_RGB16F_ARB or iformat == GL_RGB32F_ARB:
			format = GL_RGB
			type = GL_FLOAT
		elif iformat == GL_RGBA16F_ARB or iformat == GL_RGBA32F_ARB:
			format = GL_RGBA
			type = GL_FLOAT


		attachment = getattr(OpenGL.GL, 'GL_COLOR_ATTACHMENT%d_EXT' % len(self.__textures))

		id = glGenTextures(1)
		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, self.__frameid)
		glBindTexture(GL_TEXTURE_2D, id)
		glTexImage2D(GL_TEXTURE_2D, 0, iformat,  self.__width, self.__height, 0, format, type, None)
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
		glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, attachment, GL_TEXTURE_2D, id, 0)

		self.__textures.append(id)
		self.__buffers.append(attachment)
