import os
from OpenGL.GL import *
from OpenGL.GL.ARB.shader_objects import *

class Shader:
	def __init__(self, vsfile, fsfile):
		self.__program = glCreateProgram()

		self.__vertex = glCreateShader(GL_VERTEX_SHADER)
		glShaderSource(self.__vertex, open(vsfile).read())
		glCompileShader(self.__vertex)
		glAttachShader(self.__program, self.__vertex)

		self.__fragment = glCreateShader(GL_FRAGMENT_SHADER)
		glShaderSource(self.__fragment, open(fsfile).read())
		glCompileShader(self.__fragment)
		glAttachShader(self.__program, self.__fragment)

		glLinkProgram(self.__program)

	def __del__(self):
		glDetachShader(self.__program, self.__vertex)
		glDetachShader(self.__program, self.__fragment)
		glDeleteShader(self.__vertex)
		glDeleteShader(self.__fragment)
		glDeleteProgram(self.__program)


	def use(self):
		glUseProgram(self.__program)


	def update_uniform(self, name, val):
		self.use()
		loc = glGetUniformLocation(self.__program, name)
		if type(val) is int:
			glUniform1i(loc, val)
		elif type(val) is float:
			glUniform1f(loc, val)


