from OpenGL.GL import *
from OpenGL.GL.ARB.vertex_buffer_object import *
from OpenGL.raw.GL.constants import *

class VBO():
	def __init__(self):
		self.__vertid = 0
		self.__vertlen = 0
		self.__texid = 0
		self.__texlen = 0
		self.__normid = 0
		self.__numelements = 0

	def __del__(self):
		if glIsBuffer(self.__vertid):
			glDeleteBuffers(1, self.__vertid)
		if glIsBuffer(self.__texid):
			glDeleteBuffers(1, self.__texid)
		if glIsBuffer(self.__normid):
			glDeleteBuffers(1, self.__normid)


	def set_vertices(self, vertices, usage):
		if glIsBuffer(self.__vertid):
			glDeleteBuffers(1, self.__vertid)
		self.__vertid = glGenBuffersARB(1) #XXX: Non ARB version doesn't work

		glBindBuffer(GL_ARRAY_BUFFER, self.__vertid)
		glBufferDataARB(GL_ARRAY_BUFFER, vertices, usage)

		self.__vertlen = len(vertices[0])
		self.__numelements = len(vertices)


	def set_texcoords(self, texcoords, usage):
		if glIsBuffer(self.__texid):
			glDeleteBuffers(1, self.__texid)
		self.__texid = glGenBuffersARB(1) #XXX: Non ARB version doesn't work

		glBindBuffer(GL_ARRAY_BUFFER, self.__texid)
		glBufferDataARB(GL_ARRAY_BUFFER, texcoords, usage)

		self.__texlen = len(texcoords[0])


	def set_normals(self, normals, usage):
		if glIsBuffer(self.__normid):
			glDeleteBuffers(1, self.__normid)
		self.__normid = glGenBuffersARB(1) #XXX: Non ARB version doesn't work

		glBindBuffer(GL_ARRAY_BUFFER, self.__normid)
		glBufferDataARB(GL_ARRAY_BUFFER, normals, usage)


	def draw(self):
		glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT)

		glBindBuffer(GL_ARRAY_BUFFER, self.__vertid)
		glVertexPointer(self.__vertlen, GL_FLOAT, 0, None)
		glEnableClientState(GL_VERTEX_ARRAY)
	
		glBindBuffer(GL_ARRAY_BUFFER, self.__texid)
		glTexCoordPointer(self.__texlen, GL_FLOAT, 0, None)
		glEnableClientState(GL_TEXTURE_COORD_ARRAY)

		glBindBuffer(GL_ARRAY_BUFFER, self.__normid)
		glNormalPointer(GL_FLOAT, 0, None)
		glEnableClientState(GL_NORMAL_ARRAY)

		glDrawArrays(GL_QUADS, 0, self.__numelements)

		glPopClientAttrib()
